c
c ------------------------------------------------------------
c
      double precision function fpT_relax(p0)
      implicit double precision (a-h,o-z)
      parameter(tolz= 1.0d-12)
      common /mfluid/ geos(10),an(10),bn(10),cn(10),
     &        dn(10),en(10),rref(10),c2ref(10),tref(10),
     &        cv(10),emu(10),rhocav(10),pcav(10),
     &        ccav(10),rhosat(10),psat(10),csat(10),
     &        rhovm1(10),rhovm2(10),psi(10),zf(10),zv(10),
     &        mwoods,mphase
      common /trelx0/ rhoa0,rhob0,Y10,Y20,
     &        tau0,e0,temp00
      common /trelx1/ tau_iter(10),temp0,
     &        zfa0,zfb0,e1,e2
      common /fetmp0/ p00,tauloc(10),elocal(10)
      external fetemp,dfetemp
c
c     # compute isobaric-isothermal equilibrium solution
c     # iterative splitting-step approach
c
      grue1 = geos(1)-1.d0
      grue2 = geos(2)-1.d0

c     # reference pressure
      p00 = p0
c
c     # compute internal energy preserving 
c     # equilibrium temperature
      temp0 = zero1d_aux(temp00,fetemp,dfetemp,tolz,
     &                    iflag)
c
c     # compute phasic specific volume
c     # phase 1: linearized Mie-Gruneisen EOS
      e1   = cv(1)*temp0*(p0+geos(1)*bn(1))/
     &       (p0+bn(1))+cn(1)
      tau1 = grue1*cv(1)*temp0/(p0+bn(1))
      tau_iter(1) = tau1
c
c     # phase 2: linearized Mie-Gruneisen EOS
      e2   = cv(2)*temp0*(p0+geos(2)*bn(2))/
     &       (p0+bn(2))+cn(2)
      tau2 = grue2*cv(2)*temp0/(p0+bn(2))
      tau_iter(2) = tau2
c
c     # saturation equation for mechanical equilibrium
      fpT_relax = Y10*tau1+Y20*tau2-tau0
c
      zfa0 = rhoa0*tau1
      zfb0 = rhob0*tau2 
c
      if (zfa0 .lt. 0.d0 .or.
     &    zfb0 .lt. 0.d0) then
          write(66,*) 'fpTrelax=',fpTrelax,', p0=',p0
          write(66,*) 'temp0=',temp0,', temp00=',temp00
          write(66,*) 'tau1=',tau1,', tau2=',tau2
          write(66,*) 'zfa0=',zfa0,', zfb0=',zfb0,
     &             zfa0+zfb0
          write(66,*) '-----------------'
      endif
c
c     # update temperature for next iterate step
c      temp00 = temp0
      return
c
      end
c
c ------------------------------------------------------------
c
      double precision function dfpT_relax(p0)
      implicit double precision (a-h,o-z)
      common /mfluid/ geos(10),an(10),bn(10),cn(10),
     &        dn(10),en(10),rref(10),c2ref(10),tref(10),
     &        cv(10),emu(10),rhocav(10),pcav(10),
     &        ccav(10),rhosat(10),psat(10),csat(10),
     &        rhovm1(10),rhovm2(10),psi(10),zf(10),zv(10),
     &        mwoods,mphase
      common /trelx0/ rhoa0,rhob0,Y10,Y20,
     &        tau0,e0,temp00
      common /trelx1/ tau_iter(10),temp0,
     &        zfa0,zfb0,e1,e2
c
c     # derivative with respect to pressure for
c     # saturation equation
c
c     # phase 1: linearized Mie-Gruneisen EOS
      de1_dp = -tau_iter(1)*bn(1)/(p0+bn(1))
      de1_dT = (e1-cn(1))/temp0  
c
c     # phase 2: linearized Mie-Gruneisen EOS
      de2_dp = -tau_iter(2)*bn(2)/(p0+bn(2))
      de2_dT = (e2-cn(2))/temp0  
c
c     # phase 1: linearized Mie-Gruneisen EOS
      dtau1_dp = -tau_iter(1)/(p0+bn(1))
      dtau1_dT = tau_iter(1)/temp0  
      dtau1    = dtau1_dp+dtau1_dT*dTdp
c
c     # phase 2: linearized Mie-Gruneisen EOS
      dtau2_dp = -tau_iter(2)/(p0+bn(2))
      dtau2_dT = tau_iter(2)/temp0  
      dtau2    = dtau2_dp+dtau2_dT*dTdp
c
      dfpT_relax = Y10*dtau1+Y20*dtau2
c
c      write(66,*) 'dftrelax=',dftrelax
c      write(66,*) 'p0=',p0,', temp0=',temp0
c      write(66,*) 'Y10=',Y10,', Y20=',Y20,', Y30=',Y30
c      write(66,*) 'tau1=',tau_iter(1),', tau2=',tau_iter(2)
c      write(66,*) 'dtau1=',dtau1,', dtau2=',dtau2
c      write(66,*) 'dTdp=',dTdp
      return
c
      end
c
c ----------------------------------------------------------------------
c
      double precision function fetemp(temp0)
      implicit double precision (a-h,o-z)
      common /mfluid/ geos(10),an(10),bn(10),cn(10),
     &        dn(10),en(10),rref(10),c2ref(10),tref(10),
     &        cv(10),emu(10),rhocav(10),pcav(10),
     &        ccav(10),rhosat(10),psat(10),csat(10),
     &        rhovm1(10),rhovm2(10),psi(10),zf(10),zv(10),
     &        mwoods,mphase
      common /trelx0/ rhoa0,rhob0,Y10,Y20,tau0,e0,temp00
      common /fetmp0/ p0,tauloc(10),elocal(10)
c
c     # internal energy preserving condition
c
c     # compute phasic specific internal energy
c     # phase 1: linearized Mie-Gruneisen EOS
      e1 = cv(1)*temp0*(p0+geos(1)*bn(1))/(p0+bn(1))+cn(1)
      elocal(1) = e1
c
c     # phase 2: linearized Mie-Gruneisen EOS
      e2 = cv(2)*temp0*(p0+geos(2)*bn(2))/(p0+bn(2))+cn(2)
      elocal(2) = e2
c
      fetemp = Y10*e1+Y20*e2-e0
c
c      write(66,*) 'fetemp=',fetemp
c      write(66,*) 'p0=',p0,', temp0=',temp0
c      write(66,*) 'e1=',e1,', e2=',e2
c      write(66,*) '-------------------------'
      return
c
      end
c
c ------------------------------------------------------------------------
c
      double precision function dfetemp(temp0)
      implicit double precision (a-h,o-z)
      common /mfluid/ geos(10),an(10),bn(10),cn(10),
     &        dn(10),en(10),rref(10),c2ref(10),tref(10),
     &        cv(10),emu(10),rhocav(10),pcav(10),
     &        ccav(10),rhosat(10),psat(10),csat(10),
     &        rhovm1(10),rhovm2(10),psi(10),zf(10),zv(10),
     &        mwoods,mphase
      common /trelx0/ rhoa0,rhob0,Y10,Y20,tau0,e0,temp00
      common /fetmp0/ p0,tauloc(10),elocal(10)
c
c     # derivative of internal energy preserving condition
c     # with respect to temperature
c
c     # compute phasic specific internal energy
c     # phase 1: linearized Mie-Gruneisen EOS
      e1  = elocal(1)
      de1 = (e1-cn(1))/temp0
c
c     # phase 2: linearized Mie-Gruneisen EOS
      e2  = elocal(2)
      de2 = (e2-cn(2))/temp0
c
      dfetemp = Y10*de1+Y20*de2
c
c      write(66,*) 'dfetemp=',dfetemp
c      write(66,*) 'temp=',temp0,', p0=',p0
c      write(66,*) 'e1=',e1,', e2=',e2
c      write(66,*) 'de1=',de1,', de2=',de2
      return
c
      end
