c
c ------------------------------------------------------------
c
      double precision function fpTg_relax(p0)
      implicit double precision (a-h,o-z)
      parameter(tolz=1.0d-12)                  
      parameter(error_flag=-12345678.d0)
      common /mfluid/ geos(10),an(10),bn(10),cn(10),
     &        dn(10),en(10),rref(10),c2ref(10),tref(10),
     &        cv(10),emu(10),rhocav(10),pcav(10),
     &        ccav(10),rhosat(10),psat(10),csat(10),
     &        rhovm1(10),rhovm2(10),psi(10),zf(10),zv(10),
     &        mwoods,mphase
      common /mrelx0/ tau0,p00,e0,S0,temp00,Y10,Y20
      common /mrelx1/ tau_iter(10),pp0,temp0,Y1,Y2,e1,e2
      common /mrelx2/ h0,h1,h2
      common /ftemp_satu0/ p0_satu
      external ftemp_satu,dftemp_satu
c
c     # equilibrium equation for mass fraction
c
      cp1 = geos(1)*cv(1)
      cp2 = geos(2)*cv(2)
c
c     # compute saturation temperature
      p0_satu = p0
      temp0   = zero1d(temp00,ftemp_satu,dftemp_satu,tolz,
     &                 iflag)
      if (iflag .ne. 1) then
          write(66,*) 'error in pTg_relax'
          write(66,*) 'p0=',p0,', temp0=',temp0
          stop   
      endif
c
      temp00 = temp0
c
      h0 = e0+p0*tau0
c
c     # phase 1: linearized Mie-Gruneisen EOS
      e1   = e_pT_mg0(p0,temp0,1)
      tau1 = 1.d0/rho_pT_mg0(p0,temp0,1)      
      h1   = cp1*temp0+cn(1) 
      tau_iter(1) = tau1
c
c     # phase 2: linearized Mie-Gruneisen EOS
      e2   = e_pT_mg0(p0,temp0,2)
      tau2 = 1.d0/rho_pT_mg0(p0,temp0,2)      
      h2   = cp2*temp0+cn(2) 
      tau_iter(2) = tau2
c
      Y1tau = (tau0-tau2)/(tau1-tau2)
      Y2tau = (tau0-tau1)/(tau2-tau1)
      Y1h   = (h0-h2)/(h1-h2)
      Y2h   = (h0-h1)/(h2-h1)
c
      fpTg_relax = Y1tau-Y1h
c
      Y1 = 0.5d0*(Y1tau+Y1h)
      Y2 = 0.5d0*(Y2tau+Y2h)
c
c      write(66,*) 'fpTg_relax=',fpTg_relax,p0,temp0
c      write(66,*) 'Y1tau=',Y1tau,Y1h
      return
c
      end
c
c ------------------------------------------------------------
c
      double precision function dfpTg_relax(p0)
      implicit double precision (a-h,o-z)
      common /mfluid/ geos(10),an(10),bn(10),cn(10),
     &        dn(10),en(10),rref(10),c2ref(10),tref(10),
     &        cv(10),emu(10),rhocav(10),pcav(10),
     &        ccav(10),rhosat(10),psat(10),csat(10),
     &        rhovm1(10),rhovm2(10),psi(10),zf(10),zv(10),
     &        mwoods,mphase
      common /mrelx0/ tau0,p00,e0,S0,temp00,Y10,Y20
      common /mrelx1/ tau_iter(10),pp0,temp0,Y1,Y2,e1,e2
      common /mrelx2/ h0,h1,h2
c
c     # derivative of
c     # equilibrium equation for mass fraction    
c     # with respect to pressure
c
      cp1 = geos(1)*cv(1)
      cp2 = geos(2)*cv(2)
c
      tau1 = tau_iter(1)
      tau2 = tau_iter(2)
      S1   = entropy_pT_mg0(p0,temp0,1)
      S2   = entropy_pT_mg0(p0,temp0,2)
c
      dtau0 = 0.d0
      dtau1 = -tau1/(p0+bn(1))
      dtau2 = -tau2/(p0+bn(2))
      dh0   = tau0
      dh1   = 0.d0
      dh2   = 0.d0
      dY1tp = ((tau1-tau2)*(dtau0-dtau2)-
     &        (tau0-tau2)*(dtau1-dtau2))/
     &        (tau1-tau2)**2
      dY1hp = ((h1-h2)*(dh0-dh2)-
     &        (h0-h2)*(dh1-dh2))/
     &        (h1-h2)**2
c
      dtau0 = 0.d0
      dtau1 = tau1/temp0     
      dtau2 = tau2/temp0     
      dh0   = 0.d0 
      dh1   = cp1   
      dh2   = cp2  
      dY1tT = ((tau1-tau2)*(dtau0-dtau2)-
     &        (tau0-tau2)*(dtau1-dtau2))/
     &        (tau1-tau2)**2
      dY1hT = ((h1-h2)*(dh0-dh2)-
     &        (h0-h2)*(dh1-dh2))/
     &        (h1-h2)**2
c
c     # Slope of saturation curve satisfies
c     # Clausius-Clapeyron equation 
      dTdp   = (tau2-tau1)/(S2-S1) 
      dY1tau = dY1tp+dY1tT*dTdp
      dY1h   = dY1hp+dY1hT*dTdp
c
      dfpTg_relax = dY1tau-dY1h
      return
c
      end
