c
c-----------------------------------------------------------------
c
      subroutine rp_solver(ql_state,qr_state,aux1,aux2,
     &           delta,s_local,wave_local,qrp_local,
     &           dtdx,meqn)
      implicit double precision (a-h,o-z)
      dimension ql_state(20),qr_state(20)
      dimension aux1(20),aux2(20)
      dimension s_local(20),wave_local(20,20)
      dimension delta(20)
      dimension flux1(20),flux2(20)
      dimension qhll(20),qml(20),qmr(20)
      dimension work1(20),work2(20)
      dimension qrp_local(20)
c
c     # 6-equation model with partial total energy
c     # HLLC approximate Riemann solver
c
      call ctomfd(ql_state,rhoa1,rhob1,vx1,rhoh1,p1,
     &     pa1,pb1,temp1,zeta1,zfa1,zfb1,c2a1,c2b1,c21)
      call ctomfd(qr_state,rhoa2,rhob2,vx2,rhoh2,p2,
     &     pa2,pb2,temp2,zeta2,zfa2,zfb2,c2a2,c2b2,c22)
c
      if (c21 .lt. 0.d0 .or. c22 .lt. 0.d0) then
          write(6,*) 'entered HLLC'
          write(6,*) 'c21=',c21,', c22=',c22
          write(6,*) 'rhoa1=',rhoa1,', rhob1=',rhob1
          write(6,*) 'rhoa2=',rhoa2,', rhob2=',rhob2
          write(6,*) 'pa1=',pa1,', pb1=',pb1
          write(6,*) 'pa2=',pa2,', pb2=',pb2
          write(6,*) 'p1=',p1,', p2=',p2
          write(6,*) 'vx1=',vx1,', vx2=',vx2
          write(6,*) 'zfa1=',zfa1,', zfa2=',zfa2
          write(6,*) 'zfb1=',zfb1,', zfb2=',zfb2
          stop
      endif
c
c     # Davis' wave speed estimate
      s_local(1) = dmin1(vx1-dsqrt(c21),vx2-dsqrt(c22))
      s_local(3) = dmax1(vx1+dsqrt(c21),vx2+dsqrt(c22))
c
c     # compute fluxes
      flux1(1) = ql_state(1)*vx1
      flux1(2) = ql_state(2)*vx1
      flux1(3) = ql_state(3)*vx1+p1
      flux1(4) = (ql_state(4)+pa1)*vx1
      flux1(5) = (ql_state(5)+pb1)*vx1
      flux1(6) = 0.0d0
c
      flux2(1) = qr_state(1)*vx2
      flux2(2) = qr_state(2)*vx2
      flux2(3) = qr_state(3)*vx2+p2
      flux2(4) = (qr_state(4)+pa2)*vx2
      flux2(5) = (qr_state(5)+pb2)*vx2
      flux2(6) = 0.0d0
c
c     # compute hll state
      do m=1,meqn
         qhll(m) = ((s_local(3)*qr_state(m)-
     &         s_local(1)*ql_state(m))-
     &         (flux2(m)-flux1(m)))/(s_local(3)-s_local(1))
         enddo
c
c     # compute midstate velocity
      s_local(2) = qhll(3)/(qhll(1)+qhll(2))
c
      do m=1,meqn
         work1(m) = s_local(1)*ql_state(m)-flux1(m)
         work2(m) = s_local(3)*qr_state(m)-flux2(m)
         enddo
c
c     # compute mid-state pressure
      work3 = s_local(1)*ql_state(1)*vx1-
     &        (ql_state(1)*vx1**2+pa1)
      work4 = s_local(1)*ql_state(2)*vx1-
     &        (ql_state(2)*vx1**2+pb1)
      pma1  = s_local(2)*work1(1)-work3
      pmb1  = s_local(2)*work1(2)-work4
c
      work3 = s_local(3)*qr_state(1)*vx2-
     &        (qr_state(1)*vx2**2+pa2)
      work4 = s_local(3)*qr_state(2)*vx2-
     &        (qr_state(2)*vx2**2+pb2)
      pma2  = s_local(2)*work2(1)-work3
      pmb2  = s_local(2)*work2(2)-work4
c
c     # set middle state (left)
      qml(1) = work1(1)/(s_local(1)-s_local(2))
      qml(2) = work1(2)/(s_local(1)-s_local(2))
      qml(3) = (qml(1)+qml(2))*s_local(2)
      qml(4) = (work1(4)+s_local(2)*pma1)/(s_local(1)-s_local(2))
      qml(5) = (work1(5)+s_local(2)*pmb1)/(s_local(1)-s_local(2))
      qml(6) = ql_state(6)
      qml(7) = ql_state(7)
      qml(8) = ql_state(8)
      rhoma1 = qml(1)/qml(6)
      rhomb1 = qml(2)/qml(7)
c
c     # set middle state (right)
      qmr(1) = work2(1)/(s_local(3)-s_local(2))
      qmr(2) = work2(2)/(s_local(3)-s_local(2))
      qmr(3) = (qmr(1)+qmr(2))*s_local(2)
      qmr(4) = (work2(4)+s_local(2)*pma2)/(s_local(3)-s_local(2))
      qmr(5) = (work2(5)+s_local(2)*pmb2)/(s_local(3)-s_local(2))
      qmr(6) = qr_state(6)
      qmr(7) = qr_state(7)
      qmr(8) = qr_state(8)
      rhoma2 = qmr(1)/qmr(6)
      rhomb2 = qmr(2)/qmr(7)
c
      do m=1,meqn
         wave_local(m,1) = qml(m)-ql_state(m)
         wave_local(m,2) = qmr(m)-qml(m)
         wave_local(m,3) = qr_state(m)-qmr(m)
         enddo
c
c     # determine cell-edge pressure
      if (s_local(2) .gt. 0.d0) then
          zfa0 = zfa1
          zfb0 = zfb1
          zf0_color = ql_state(8)
c
          if (s_local(1) .lt. 0.d0) then
              pma0 = pma1
              pmb0 = pmb1
c
              rhoma0 = rhoma1
              rhomb0 = rhomb1
          else
              pma0 = pa1
              pmb0 = pb1
c
              rhoma0 = rhoa1/zfa1
              rhomb0 = rhob1/zfb1
          endif
      elseif (s_local(2) .lt. 0.d0) then
          zfa0 = zfa2
          zfb0 = zfb2
          zf0_color = qr_state(8)
c
          if (s_local(3) .gt. 0.d0) then
              pma0 = pma2
              pmb0 = pmb2
c
              rhoma0 = rhoma2
              rhomb0 = rhomb2
          else
              pma0 = pa2
              pmb0 = pb2
c
              rhoma0 = rhoa2/zfa2
              rhomb0 = rhob2/zfb2
          endif
      else
          zfa0 = 0.5d0*(zfa1+zfa2)
          zfb0 = 0.5d0*(zfb1+zfb2)
          zf0_color = 0.5d0*(ql_state(8)+qr_state(8))
c
          pma0 = 0.5d0*(pma1+pma2)
          pmb0 = 0.5d0*(pmb1+pmb2)
c
          rhoma0 = 0.5d0*(rhoma1+rhoma2)
          rhomb0 = 0.5d0*(rhomb1+rhomb2)
      endif

      
      qrp_local(1) = s_local(2)
      qrp_local(2) = pma0
      qrp_local(3) = pmb0
      qrp_local(4) = rhoma0
      qrp_local(5) = rhomb0
      qrp_local(6) = zfa0
      qrp_local(7) = zfb0
      qrp_local(8) = zf0_color
c
c      pm1 = pma1+pmb1
c      pm2 = pma2+pmb2
c      write(77,*) 'pm1=',pm1,', pm2=',pm2
c      write(77,*) 'pma1=',pma1,', pmb1=',pmb1
c      write(77,*) 'pma2=',pma2,', pmb2=',pmb2
c      write(77,*) 'ql=',(ql_state(m),m=1,meqn)
c      write(77,*) 'qr=',(qr_state(m),m=1,meqn)
c      write(77,*) 'qml=',(qml(m),m=1,meqn)
c      write(77,*) 'qmr=',(qmr(m),m=1,meqn)
c      write(77,*) 's=',(s_local(mw),mw=1,3)
c      do mw=1,3
c         write(77,*) 'mw=',mw
c         write(77,*) (wave_local(m,mw),m=1,meqn)
c         enddo
      return
c
      end
