c
c ------------------------------------------------------------
c
      subroutine pTg_solver(qpTg_relax,qloc,meqn,dt,
     &           is_admissible)
      implicit double precision (a-h,o-z)
      parameter(error_flag=-12345678.d0)
      parameter(itmax=12,c0=0.5d0,alf0=1.0d-6)
      common /mfluid/ geos(10),an(10),bn(10),cn(10),
     &        dn(10),en(10),rref(10),c2ref(10),tref(10),
     &        cv(10),emu(10),rhocav(10),pcav(10),
     &        ccav(10),rhosat(10),psat(10),csat(10),
     &        rhovm1(10),rhovm2(10),psi(10),zf(10),zv(10),
     &        mwoods,mphase
      common /eos_local/ rhoe10,rhoe20,
     &        temp10,temp20,S10,S20,
     &        gb10,gb20,zeta10,zeta20
      common /mrelx0/ tau0,p00,e0,S0,temp00,Y10,Y20
      common /mrelx1/ tau_ptg(10),pptg,temptg,Y1ptg,Y2ptg,
     &        e1ptg,e2ptg
      common /pt0/ dzf,zfmin,vofeps,tolm,tempeps
      dimension qpTg_relax(20),qloc(20)
      external fpTg_relax,dfpTg_relax
c
c     # 6-equation model
c     # Gibbs free energy relaxation
c     # iterative solver: hybrid secant-Newton method
c
      is_admissible = 1
c
      do m=1,meqn
         qpTg_relax(m) = qloc(m)
         enddo
c
      call ctomfd(qloc,rhoa0,rhob0,vx0,
     &     rhoh0,p0,pa0,pb0,temp0,zeta0,
     &     zfa0,zfb0,c2a0,c2b0,c20)
c
      tau0 = 1.d0/(rhoa0+rhob0)
      Y10  = rhoa0/(rhoa0+rhob0)
      Y20  = rhob0/(rhoa0+rhob0)
      e0   = (zfa0*rhoe10+zfb0*rhoe20)/(rhoa0+rhob0)
      S0   = Y10*S10+Y20*S20
      p00  = p0
      temp00 = temp0
c
c     # initial guess
      ptg0 = p0
c
c     # solve for equilibrium pressure    
c     # hybrid safeguard Newton and secant method
      ptg1 = ptg0
      ptg2 = ptg0
      fpTg1 = fpTg_relax(ptg1)
      if (dabs(fpTg1) .lt. tolm) then
c          ptg = ptg1
          go to 99
      elseif (fpTg1 .eq. error_flag) then
          is_admissible = -1
          go to 99
      endif
c
      alf    = 1.0d0
      iter   = 0
      dfpTg1 = dfpTg_relax(ptg1)
c
 19   continue
      if ((iter .gt. itmax) .or.
     &    (alf .lt. alf0)) then
          is_admissible = -2
          go to 99
      endif
c
c     # Newton's iteration
      ptgn = ptg1-alf*fpTg1/dfpTg1
c
      if (ptgn .lt. pcav(1)) then
c         # safeguard step size reduction
c
          alf = c0*alf
          go to 19 
      else
          fpTgn = fpTg_relax(ptgn)
          if (dabs(fpTgn) .lt. tolm) then
              ptg = ptgn
              go to 39
          elseif (fpTgn .eq. error_flag) then
              alf = c0*alf
              go to 19
          endif
c
          if (fpTg1*fpTgn .le. 0.d0) then
              ptg2 = ptgn
              go to 29
          else
              ptg1   = ptgn
              ptg2   = ptgn
              fpTg1  = fpTgn
              dfpTg1 = dfpTg_relax(ptg2)
              alf    = 1.d0
              iter   = iter+1
              go to 19
          endif
      endif
c
 29   continue
      ptg = zeroin(ptg1,ptg2,fpTg_relax,tolm)
c
      if (ptg .eq. error_flag) then
          is_admissible = -3
          go to 99
      endif
c
 39   continue
      Eg1 = e1ptg+0.5d0*vx0**2
      Eg2 = e2ptg+0.5d0*vx0**2
c
c     # reinitializ state variables
      qpTg_relax(1) = Y1ptg/tau0
      qpTg_relax(2) = Y2ptg/tau0
      qpTg_relax(4) = qpTg_relax(1)*Eg1
      qpTg_relax(5) = qpTg_relax(2)*Eg2
      qpTg_relax(6) = qpTg_relax(1)*tau_ptg(1)
      qpTg_relax(7) = qpTg_relax(2)*tau_ptg(2)
c
      if (qpTg_relax(1) .lt. 0.d0 .or.
     &    qpTg_relax(2) .lt. 0.d0) then
c         # non-admissible state
c
          is_admissible = 2
          gb1_ptg = gibbs_pT_mg0(ptg,temptg,1)
          gb2_ptg = gibbs_pT_mg0(ptg,temptg,2)
          go to 99
      endif
c
 99   continue
      if (is_admissible .ne. 1) then
          write(66,*) 'error in pTg_solver: is_admissible=',
     &                is_admissible
c
          write(66,*) 'ptg=',ptg,', temptg=',temptg
          write(66,*) 'Y1ptg=',Y1ptg,', Y2ptg=',Y2ptg
          write(66,*) 'iter=',iter,', alf=',alf
c          write(66,*) 'qloc=',(qloc(m),m=1,meqn)
c          write(66,*) 'qpTg_relax=',(qpTg_relax(m),m=1,meqn)
          write(66,*) 'p0=',p0,', temp0=',temp0
          write(66,*) 'gb10=',gb10,', gb20=',gb20,gb10-gb20
          write(66,*) 'gb1_ptg=',gb1_ptg,', gb2_ptg=',gb2_ptg
          write(66,*) 'Y10=',Y10
          write(66,*) '----------------'
c          stop
       endif
      return
c
      end
