c
c -----------------------------------------------------------------
c
      double precision function satu_temp(p0,temp0)
      implicit double precision (a-h,o-z)
      parameter(error_flag=-12345678.d0)
      parameter(itmax=36,c0=0.5d0,alf0=1.0d-6)
      parameter(tolz = 1.0d-8)
      common /gb0/ p00
      common /Temp_cr/ temp_crmin,temp_crmax
      external fgb,dfgb
c
c     # compute saturation temperature
c     # hybrid safeguard Newton and secant method
c
c     # reference pressure
      p00 = p0
c
      temp1 = temp0
      temp2 = temp1
      fgb1 = fgb(temp1)
      if (dabs(fgb1) .lt. tolz) then
          satu_temp = temp0
          return
      elseif (fgb1 .eq. error_flag) then
c          write(6,*) 'error in satu_temp'
c          write(6,*) 'p0 =',p0,', temp0=',temp0
c
          satu_temp = error_flag
          return
      endif 
c
      dfgb1 = dfgb(temp1)
c
      alf  = 1.0d0
      iter = 0
c
 19   continue
      if ((iter .gt. itmax) .or.
     &    (alf .lt. alf0)) then
c          write(66,*) 'satu_temp failed to converge'
c          write(66,*) 'p0=',p0,', temp0=',temp0
c     &                'dfgb1=',dfgb1  
c
          satu_temp = error_flag
          return
      endif
c
c     # safeguard Newton's iteration
      tempn = temp1-alf*fgb1/dfgb1
c
      if ((tempn-temp_crmin)*(tempn-temp_crmax) .gt. 0.d0) then
c         # safeguard step size reduction
c
          alf = c0*alf
          go to 19
      else
          fgbn = fgb(tempn)
          if (dabs(fgbn) .lt. tolz) then
              satu_temp = tempn
              return
          elseif (fgbn .eq. error_flag) then
              alf = c0*alf
              go to 19
          endif
c
          if (fgb1*fgbn .le. 0.d0) then
              temp2 = tempn
              go to 29
          else
              temp1 = tempn
              temp2 = tempn
              fgb1  = fgbn
              dfgb1 = dfgb(temp2)
              alf   = 1.d0
              iter  = iter+1
              go to 19
          endif
      endif
c
 29   continue
      satu_temp = zeroin(temp1,temp2,fgb,tolz)
      return
c
      end
c
c ---------------------------------------------------------------------
c
      double precision function fgb(temp)
      implicit double precision (a-h,o-z)
      parameter(error_flag=-12345678.d0)
      common /mfluid/ geos(10),an(10),bn(10),cn(10),
     &        dn(10),en(10),rref(10),c2ref(10),tref(10),
     &        cv(10),emu(10),rhocav(10),pcav(10),
     &        ccav(10),rhosat(10),psat(10),csat(10),
     &        rhovm1(10),rhovm2(10),psi(10),zf(10),zv(10),
     &        mwoods,mphase
      common /gb0/ p0
      common /Temp_cr/ temp_crmin,temp_crmax
c
c     # equal Gibbs free energy
c
      if ((temp-temp_crmin)*(temp-temp_crmax) .gt. 0.d0) then
          write(66,*) 'error in fgb: temp=',temp
c          write(66,*) 'temp_crmin=',temp_crmin,
c     &               ', temp_crmin=',temp_crmax
          fgb = error_flag
          return
      endif
c
      grue1 = geos(1)-1.d0
      grue2 = geos(2)-1.d0
      cp1   = geos(1)*cv(1)
      cp2   = geos(2)*cv(2)
c
c     # Gibbs free energy: G(p,T) = h(T)-TS(p,T)
      h1  = cp1*temp+cn(1)
      S1  = cv(1)*dlog(temp**geos(1)/(p0+bn(1))**grue1)+dn(1)
      gb1 = h1-temp*S1
c
      h2  = cp2*temp+cn(2)
      S2  = cv(2)*dlog(temp**geos(2)/(p0+bn(2))**grue2)+dn(2)
      gb2 = h2-temp*S2
c
      fgb = (gb2-gb1)/dmax1(dabs(gb1),dabs(gb2))
c
c      fgb = gb2-gb1
c      
c      write(66,*) 'fgb=',fgb,temp,gb1,gb2
      return
c
      end
c
c ------------------------------------------------------------------------
c
      double precision function dfgb(temp)
      implicit double precision (a-h,o-z)
      common /mfluid/ geos(10),an(10),bn(10),cn(10),
     &        dn(10),en(10),rref(10),c2ref(10),tref(10),
     &        cv(10),emu(10),rhocav(10),pcav(10),
     &        ccav(10),rhosat(10),psat(10),csat(10),
     &        rhovm1(10),rhovm2(10),psi(10),zf(10),zv(10),
     &        mwoods,mphase
      common /gb0/ p0
c
c     # first-derivative of the function of
c     # equal Gibbs free energy
c
      grue1 = geos(1)-1.d0
      grue2 = geos(2)-1.d0
      cp1   = geos(1)*cv(1)
      cp2   = geos(2)*cv(2)
c
      h1   = cp1*temp+cn(1)
      dh1  = cp1
      S1   = cv(1)*dlog(temp**geos(1)/(p0+bn(1))**grue1)+dn(1)
      dS1  = cp1/temp
      gb1  = h1-temp*S1
c     # this gives specific entropy for phase 1
      dgb1 = dh1-(temp*dS1+S1)
c
      h2   = cp2*temp+cn(2)
      dh2  = cp2
      S2   = cv(2)*dlog(temp**geos(2)/(p0+bn(2))**grue2)+dn(2)
      dS2  = cp2/temp
      gb2  = h2-temp*S2
c     # this gives specific entropy for phase 2
      dgb2 = dh2-(temp*dS2+S2)
c
      dfgb = (dgb2-dgb1)/dmax1(dabs(gb1),dabs(gb2))
c
c      dfgb = dgb2-dgb1
c      
c      write(66,*) 'dfgb=',dfgb,temp,dgb1,dgb2
      return
c
      end
