c
c -------------------------------------------------------------------
c
      subroutine tclaw1(maxmx,meqn,mwaves,mbc,maux,mx,q,aux,
     &           rkwork,mrkwork,
     &           xlower,dx,tstart,tend,dtv,cflv,nv,method,mthlim,
     &           mthbc,work,mwork,info,bc1,rp1,tfluct1,src1,
     &           b4step1,a4step1)
      implicit double precision (a-h,o-z)
      external bc1,rp1,tfluct1,src1,a4step1,b4step1
      dimension q(1-mbc:maxmx+mbc,meqn)
      dimension aux(1-mbc:maxmx+mbc,maux)
      dimension work(mwork),rkwork(mrkwork)
      dimension mthlim(mwaves),method(10),dtv(5),cflv(4),nv(2)
      dimension mthbc(2)
      common /comxt/ dtcom,dxcom,tcom
c
c     # solve hyperbolic system of conservation laws 
c     # in one space dimension of form
c
c     # capa*q_t+A q_x = psi
c
c     # method employs high-order WENO reconstruction and 
c     # Runge-Kutta time integration
c
c     # beginning tclaw1 code
c
      info = 0
      t    = tstart
      maxn = nv(1)
c
c     # initial dt
      dt     = dtv(1)   
      cflmax = 0.d0
      dtmin  = dt
      dtmax  = dt
      nv(2)  = 0
c
      if (mx .gt. maxmx) then
          info = 1
          go to 900
      endif
c
      if (method(1) .eq. 0) then
c         # fixed size time steps.  Compute the number of steps
          if (tend .lt. tstart) then
c             # single step mode
              maxn = 1
          else
              maxn = (tend-tstart+1d-10)/dt
              if (dabs(maxn*dt-(tend-tstart)) .gt.
     &                 1d-5*(tend-tstart)) then
c                 # dt doesn't divide time interval integer 
c                 # number of times
                  info = 2
                  go to 900
              endif
          endif
      endif
c
      if (method(1) .eq. 1 .and. cflv(2).gt.cflv(1)) then
          info = 3
          go to 900
      endif
c
c     # partition work array into pieces for passing into step1
c     # most of this could be set up inside ts_*, but that would
c     # actually require _more_ (duplicate) lines of code
c     # even though it would reduce the number of arguments to ts_*

      i0ql    = 1
      i0qr    = i0ql+(maxmx+2*mbc)*meqn
      i0auxl  = i0qr+(maxmx+2*mbc)*meqn
      i0auxr  = i0auxl+(maxmx+2*mbc)*maux
      i0wave  = i0auxr+(maxmx+2*mbc)*maux
      i0s     = i0wave+(maxmx+2*mbc)*meqn*mwaves
      i0dtdx  = i0s+(maxmx+2*mbc)*mwaves
      i0amdq  = i0dtdx+(maxmx+2*mbc)
      i0apdq  = i0amdq+(maxmx+2*mbc)*meqn
      i0amdq2 = i0apdq+(maxmx+2*mbc)*meqn
      i0apdq2 = i0amdq2+(maxmx+2*mbc)*meqn
      i0qwork = i0apdq2+(maxmx+2*mbc)*meqn
      i0awork = i0qwork+(maxmx+2*mbc)*meqn                        
      i0qrp1  = i0awork+(maxmx+2*mbc)*maux                        
      i0qrp2  = i0qrp1+(maxmx+2*mbc)*meqn                        
      i0next  = i0qrp2+(maxmx+2*mbc)*meqn 
      mused   = i0next-1                
      mwork1  = mwork-mused              
c
      if (mused .gt. mwork) then
          write(6,*) 'mwork must be increased to ',mused
          info = 4
          go to 900
      endif
c
c     # partition rkwork array into pieces for passing into ts_*
      i0qrk1 = 1
      if (method(2) .eq. 5) then
          i0qrk2  = i0qrk1+(maxmx+2*mbc)*meqn
          i0rkend = i0qrk2+(maxmx+2*mbc)*meqn-1
      else
          i0rkend = i0qrk1+(maxmx+2*mbc)*meqn-1
      endif
c
      if (mrkwork .lt. i0rkend) then
          write(6,*) 'mrkwork must be increased to ',i0rkend
          info = 4
          go to 900
      endif
c
c     # main interation loop
c
      if (maxn.eq.0) go to 900
c
      do n=1,maxn
         told = t   
         if (told+dt .gt. tend .and. 
     &       tstart  .lt. tend) dt = tend-told
c
         tcom = told
         dtcom = dt
         dxcom = dx
c
         call bc1(maxmx,meqn,mbc,mx,xlower,dx,q,maux,aux,told,dt,mthbc)
c
         call copyq1(maxmx,meqn,mbc,mx,q,work(i0qwork))
c
         call copyaux1(maxmx,maux,mbc,mx,aux,work(i0awork))
c
         go to (110,120,130) method(2)
c
c        # temporal discretization
 110     continue
c        # explicit first-order Runge-Kutta method
         call ts1_erk1(maxmx,meqn,mwaves,mbc,mx,q,
     &        work(i0qwork),aux,work(i0awork),
     &        work(i0ql),work(i0qr),work(i0auxl),work(i0auxr),
     &        xlower,dx,told,dt,method,mthlim,cflv,cfl,bc1,src1,maux,
     &        work(i0s),work(i0amdq),work(i0apdq),work(i0amdq2),
     &        work(i0apdq2),work(i0dtdx),work(i0wave),
     &        rkwork(i0qrk1),work(i0qrp1),work(i0qrp2),
     &        rp1,tfluct1,mthbc,
     &        work(i0next),mwork1)
         go to 199
c
 120     continue
c        # explicit second-order Runge-Kutta method
         call ts1_erk2(maxmx,meqn,mwaves,mbc,mx,q,
     &        work(i0qwork),aux,work(i0awork),
     &        work(i0ql),work(i0qr),work(i0auxl),work(i0auxr),
     &        xlower,dx,told,dt,method,mthlim,cflv,cfl,bc1,src1,maux,
     &        work(i0s),work(i0amdq),work(i0apdq),work(i0amdq2),
     &        work(i0apdq2),work(i0dtdx),work(i0wave),
     &        work(i0qrp1),work(i0qrp2),
     &        rkwork(i0qrk1),rp1,tfluct1,mthbc,
     &        work(i0next),mwork1)
         go to 199
c
 130     continue
c        # explicit third-order Runge-Kutta method
         call ts1_erk3(maxmx,meqn,mwaves,mbc,mx,q,
     &        work(i0qwork),aux,work(i0awork),
     &        work(i0ql),work(i0qr),work(i0auxl),work(i0auxr),
     &        xlower,dx,told,dt,method,mthlim,cflv,cfl,bc1,src1,maux,
     &        work(i0s),work(i0amdq),work(i0apdq),work(i0amdq2),
     &        work(i0apdq2),work(i0dtdx),work(i0wave),
     &        work(i0qrp1),work(i0qrp2),
     &        rkwork(i0qrk1),rp1,tfluct1,mthbc,
     &        work(i0next),mwork1)
         go to 199
c
 199     continue
c        # time at end of step
         t = told+dt
c
c        # diagnosis
         call dg1(maxmx,meqn,maux,mbc,mx,xlower,dx,t,
     &        q,work(i0qwork),aux,work(i0awork),
     &        work(i0next),mwork1)
c
         if (method(4) .eq. 1) write(6,601) n,cfl,dt,t
c
         if (method(1) .eq. 1) then
c            # choose new time step if variable time step
             if (cfl .gt. cflv(1)) then
                 write(6,*) 'cfl .gt. cflv(1): cfl=',cfl,', dt=',dt
c                 stop
c
                 call estdt(maxmx,mbc,mx,meqn,maux,
     &                dx,dt0,cflv(2),q,aux)

                 dt = dt0
             else
                 if (cfl .eq. 0.d0) then
                     dt = dtv(2)
                 else
                     dt = dmin1(dtv(2),dt*cflv(2)/cfl)
                 endif
             endif
c
             dtmin = dmin1(dt,dtmin)
             dtmax = dmax1(dt,dtmax)
         endif
c
c        # see if we are done
         nv(2) = nv(2)+1
         if (t .ge. tend) go to 900
         enddo   
c
 900   continue
c      # return information
c
       if (method(1) .eq. 1 .and. 
     &     t .lt. tend .and. nv(2) .eq. maxn) then
c          # too many timesteps
           info = 11
       endif
c
       if (method(1).eq.0 .and. cflmax .gt. cflv(1)) then
c          # Courant number too large with fixed dt
           info = 12
       endif
c
       tend    = t
       cflv(3) = cflmax
       cflv(4) = cfl
       dtv(3) = dtmin
       dtv(4) = dtmax
       dtv(5) = dt
       return 
c
 601   format('tclaw1... Step',i6,
     &                   '   Courant number =',f6.3,'  dt =',d12.4,
     &                   '  t =',d12.4)
       end
